/* Copyright 2009 Fog Creek Software, Inc. */

using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Data;
using System.Collections;

/* FogBugz namespaces-- make sure you add the neccesary assembly references to
 * DLL files contained in C:\Program Files\FogBugz\website\bin\ */
using FogCreek.FogBugz;
using FogCreek.FogBugz.Plugins;
using FogCreek.FogBugz.Plugins.Api;
using FogCreek.FogBugz.Plugins.Interfaces;
using FogCreek.FogBugz.UI;
using FogCreek.FogBugz.UI.EditableTable;
using FogCreek.FogBugz.UI.Dialog;
using FogCreek.FogBugz.Database;

namespace EditableTable_Dialog_Example
{
    /* Class Declaration: Inherit from Plugin, implement IPluginDatabase, etc */
    public class EditableTable_Dialog_Example : Plugin, IPluginDatabase, IPluginPageDisplay,
        IPluginRawPageDisplay, IPluginExtrasMenu
    {
        public const int PLUGIN_SCHEMA_VERSION = 1;
        private string sTableId;
        
        /* Constructor: We'll just initialize the inherited Plugin class, which 
         * takes the passed instance of CPluginApi and sets its "api" member variable. */
        public EditableTable_Dialog_Example(CPluginApi api)
            : base(api)
        {
        }

        #region IPluginExtrasMenu Members

        public CNavMenuLink[] ExtrasMenuLinks()
        {
            /* Create a link to this plugin's page in the "Extras" menu, but only if
             * the current user is a logged-in (normal or admin) FogBugz user */
            if (api.Person.GetCurrentPerson() != null &&
                api.Person.GetCurrentPerson().GetPermissionLevel() >= PermissionLevel.Normal)
            {
                return new CNavMenuLink[] {
                    new CNavMenuLink("Adopt a Kiwi!",api.Url.PluginPageUrl())
                };
            }
            else
                return new CNavMenuLink[] { };
        }

        #endregion

        /* Use of database tables in FogBugz plugins is covered in detail in the article,
         * "Implementing IPluginDatabase" https://developers.fogbugz.com/default.asp?W19 */
        #region IPluginDatabase Members

        public CTable[] DatabaseSchema()
        {
            /* Schema consists of a "Kiwi" table and a "Zoo" table. */
            CTable kiwiTable = api.Database.NewTable(api.Database.PluginTableName("Kiwi"));
            kiwiTable.sDesc = "A table full of kiwis.";
            kiwiTable.AddAutoIncrementPrimaryKey("ixKiwi");
            kiwiTable.AddTextColumn("sKiwiFullName", "The full name of the kiwi.");
            kiwiTable.AddIntColumn("ixZoo", false);
            kiwiTable.AddDateColumn("dtDateOfBirth", false, "The kiwi's date of birth.");
            
            CTable zooTable = api.Database.NewTable(api.Database.PluginTableName("Zoo"));
            zooTable.sDesc = "A table of zoos.";
            zooTable.AddAutoIncrementPrimaryKey("ixZoo");
            zooTable.AddTextColumn("sZooName", "The name of the zoo.");

            return new CTable[] { kiwiTable , zooTable};
        }

        public int DatabaseSchemaVersion()
        {
            return PLUGIN_SCHEMA_VERSION;
        }

        public void DatabaseUpgradeAfter(int ixVersionFrom, int ixVersionTo, 
            CDatabaseUpgradeApi apiUpgrade)
        {
            /* if this is the first time the plugin is installed, populate the zoo table */
            if (ixVersionFrom < 1)
            {
                string[] zoos = {"San Diego",
                                 "Pittsburgh",
                                 "Bronx",
                                 "London",
                                 "Mecklenburg",
                                 "Pukerua Bay" };

                for (int i = 0; i < zoos.Length; i++)
                {
                    CInsertQuery iq = api.Database.NewInsertQuery(
                        api.Database.PluginTableName("Zoo"));

                    iq.InsertString("sZooName", zoos[i]);                   
                    iq.Execute();
                }
            }
        }

        public void DatabaseUpgradeBefore(int ixVersionFrom, int ixVersionTo, 
            CDatabaseUpgradeApi apiUpgrade)
        {
        }

        #endregion

        /* Methods to created, read, update and delete kiwis in the database, using
         * fields in the plugin api request object */
        #region Kiwi CRUD

        protected void InsertKiwi()
        {
            CInsertQuery insert = api.Database.NewInsertQuery(
                api.Database.PluginTableName("Kiwi"));

            insert.InsertString("sKiwiFullName", api.Request[api.AddPluginPrefix("sKiwiFullName")].ToString());

            /* If the birth date is supplied, attempt to parse it. If successful,
             * convert from the client time zone to UTC using the plugin API */
            if (api.Request[api.AddPluginPrefix("dtDateOfBirth")] != null &&
                api.Request[api.AddPluginPrefix("dtDateOfBirth")].ToString().Length > 1)
            {
                DateTime dt;
                if (DateTime.TryParse(api.Request[api.AddPluginPrefix("dtDateOfBirth")].ToString(), out dt))
                    insert.InsertDate("dtDateOfBirth", api.TimeZone.UTCFromCTZ(dt));
            }

            if (api.Request[api.AddPluginPrefix("ixZoo")] != null)
                insert.InsertInt("ixZoo", Convert.ToInt32(api.Request[api.AddPluginPrefix("ixZoo")]));

            insert.Execute();
        }

        /* Returns a DataSet object with all the Kiwis in the database with
         * their Zoo values */
        protected DataSet FetchKiwis()
        {
            CSelectQuery select = api.Database.NewSelectQuery(
                api.Database.PluginTableName("Kiwi"));

            select.AddLeftJoin(api.Database.PluginTableName("Zoo"),
                api.Database.PluginTableName("Kiwi") + ".ixZoo = " +
                api.Database.PluginTableName("Zoo") + ".ixZoo");
            select.AddSelect("*");

            return select.GetDataSet();
        }

        protected void UpdateKiwi()
        {
            CUpdateQuery update =
                api.Database.NewUpdateQuery(api.Database.PluginTableName("Kiwi"));

            if (api.Request[api.AddPluginPrefix("sKiwiFullName")] != null)
            {
                update.UpdateString("sKiwiFullName", api.Request[api.AddPluginPrefix("sKiwiFullName")]);
            }

            if (api.Request[api.AddPluginPrefix("ixZoo")] != null)
                update.UpdateInt("ixZoo", Convert.ToInt32(api.Request[api.AddPluginPrefix("ixZoo")]));

            /* If the birth date is supplied, attempt to parse it. If successful,
             * convert from the client time zone to UTC using the plugin API.
             * If the birth date is an empty string, the user has removed it.
             * Using DateTime.MinValue will update the DB field with a null value */
            if (api.Request[api.AddPluginPrefix("dtDateOfBirth")] != null)
            {
                DateTime dt;
                if (DateTime.TryParse(api.Request[api.AddPluginPrefix("dtDateOfBirth")].ToString(), out dt))
                    update.UpdateDate("dtDateOfBirth", api.TimeZone.UTCFromCTZ(dt));
                else if (api.Request[api.AddPluginPrefix("dtDateOfBirth")].ToString().Length == 0)
                    update.UpdateDate("dtDateOfBirth", DateTime.MinValue);
                else
                {
                    /* If the user supplied a string that cannot parse to a DateTime,
                     * The plugin should abort the update and display an error to the
                     * user. To keep this example simple, the value is ignored */
                }
            }

            update.AddWhere("ixKiwi = @ixKiwi");
            update.SetParamInt("@ixKiwi", Convert.ToInt32(api.Request[api.AddPluginPrefix("ixKiwi")]));
            update.Execute();
        }

        protected void DeleteKiwi()
        {
            CDeleteQuery delete =
                api.Database.NewDeleteQuery(api.Database.PluginTableName("Kiwi"));

            if (api.Request[api.AddPluginPrefix("ixKiwi")] != null)
            {
                delete.AddWhere("ixKiwi = @ixKiwi");
                delete.SetParamInt("@ixKiwi", Convert.ToInt32(api.Request[api.AddPluginPrefix("ixKiwi")]));
                delete.Execute();
            }
        }

        #endregion

        #region IPluginPageDisplay Members

        public string PageDisplay()
        {
            string sHTML = "";
            sHTML += RenderHeader();
            sHTML += KiwiTable().RenderHtml();
            return sHTML;
        }

        public PermissionLevel PageVisibility()
        {
            /* We'll let all logged-in users with "normal" and "admin" permissions access
             * this page. */
            return PermissionLevel.Normal;
        }

        #endregion

        /* use the PluginRawPage to handle AJAX requests when the user adds, edits or
         * deletes Kiwis. */
        #region IPluginRawPageDisplay Members

        public string RawPageDisplay()
        {
            /* check for a valid action token in the request before processing to
             * prevent cross site request forgery */
            if ((api.Request[api.AddPluginPrefix("sAction")] != null) &&
                (api.Request[api.AddPluginPrefix("actionToken")] != null) &&
                api.Security.ValidateActionToken(api.Request[api.AddPluginPrefix("actionToken")]))
            {
                switch (api.Request[api.AddPluginPrefix("sAction")].ToString())
                {
                    case "new":
                        InsertKiwi();
                        break;
                    case "edit":
                        UpdateKiwi();
                        break;
                    case "delete":
                        DeleteKiwi();
                        break;
                    default:
                        return string.Empty;
                }
                /* you should really handle errors but we're keeping it simple */
                /* return the updated table as xml so FogBugz can update the page */
                api.Response.ContentType = "text/xml";
                return KiwiTable().RenderXml();
            }
            else
                return string.Empty;
        }

        public PermissionLevel RawPageVisibility()
        {
            return PermissionLevel.Normal;
        }

        #endregion

        /* This method builds the EditableTable which displays the kiwis. It includes
         * DialogTemplates for add, edit and delete dialogs */
        protected CEditableTable KiwiTable()
        {
            CEditableTable editableTableKiwis = new CEditableTable("kiwitable");
            sTableId = editableTableKiwis.sId;
            editableTableKiwis.sWidth = "640px";
            /* Define the header row of the table */
            editableTableKiwis.Header.AddCell("Edit");
            editableTableKiwis.Header.AddCell("Delete");
            editableTableKiwis.Header.AddCell("Kiwi");
            editableTableKiwis.Header.AddCell("Full Name");
            editableTableKiwis.Header.AddCell("Date of Birth");
            editableTableKiwis.Header.AddCell("Zoo of Residence");
            /* this variable means we don't need to mess with colspans later in the code */
            int nColCount = editableTableKiwis.Header.Cells.Count;

            /* Create the edit dialog template object used when the user clicks the
             * edit icon in a particular row */
            CDialogTemplate dlgTemplateEdit = DialogTemplateEdit();

            /* Create the new kiwi dialog template object used when the user clicks Add
             * New Kiwi or the add icon in the footer row */
            CDialogTemplate dlgTemplateNew = DialogTemplateNew();

            /* Create the delete dialog template object used when the user clicks the
             * delete icon in a particular row */
            CDialogTemplate dlgTemplateDelete = DialogTemplateDelete();

            /* setup a DataSet and fetch the kiwis from the database */
            DataSet dsKiwis = FetchKiwis();
            string sKiwiName = "none";
            int ixKiwi = -1;
            DateTime dtDateOfBirth;
            int ixZoo = -1;
            string sZooName = "";

            /* If the DataSet contains any rows, loop through them and populate the table
             * and dialog template data Hashtables */
            if (dsKiwis.Tables[0] != null && dsKiwis.Tables[0].Rows.Count > 0)
            {
                for (int i = 0; i < dsKiwis.Tables[0].Rows.Count; i++)
                {
                    ixKiwi = Convert.ToInt32(dsKiwis.Tables[0].Rows[i]["ixKiwi"]);
                    sKiwiName = dsKiwis.Tables[0].Rows[i]["sKiwiFullName"].ToString();
                    if (!(DateTime.TryParse(dsKiwis.Tables[0].Rows[i]["dtDateOfBirth"].ToString(),
                                            out dtDateOfBirth)))
                    {
                        dtDateOfBirth = DateTime.MinValue;
                    }
                    ixZoo = Convert.ToInt32(dsKiwis.Tables[0].Rows[i]["ixZoo"]);
                    sZooName = dsKiwis.Tables[0].Rows[i]["sZooName"].ToString();

                    /* create a new table row and set the row id to the unique ixKiwi */
                    CEditableTableRow row = new CEditableTableRow();
                    row.sRowId = ixKiwi.ToString();
                    /* CEditableTable provides standard edit and delete icon links.
                     * The second parameter is the name of the dialog to open. The fourth
                     * parameter is the URL to link to if javascript is not available.
                     * Note: we do not provide a non-javascript mode in this example. */
                    row.AddCell(CEditableTable.LinkShowDialogEditIcon(
                                    sTableId,
                                    "edit",
                                    row.sRowId,
                                    CommandUrl("edit", ixKiwi)));
                    row.AddCell(CEditableTable.LinkShowDialogDeleteIcon(
                                    sTableId,
                                    "delete",
                                    row.sRowId,
                                    CommandUrl("delete", ixKiwi)));
                    row.AddCell(ixKiwi.ToString());
                    /* make sure to run HtmlEncode on any user data before displaying it! */
                    row.AddCell(HttpUtility.HtmlEncode(sKiwiName));
                    row.AddCell((dtDateOfBirth == DateTime.MinValue) ?
                                "&nbsp;" :
                                HttpUtility.HtmlEncode(
                                    api.TimeZone.CTZFromUTC(dtDateOfBirth).ToShortDateString()
                                )
                        );
                    row.AddCell(HttpUtility.HtmlEncode(sZooName));
                    editableTableKiwis.Body.AddRow(row);

                    /* Now that the row is populated for display, put the data in a hash table
                     * to be used in populating the pop-up add, edit and delete dialogs. */
                    Hashtable hashData = new Hashtable();
                    hashData.Add("ixKiwi", ixKiwi);
                    hashData.Add("sKiwiFullName", HttpUtility.HtmlEncode(sKiwiName));
                    hashData.Add("dtDateOfBirth", dtDateOfBirth.ToShortDateString());
                    hashData.Add("sZooName", HttpUtility.HtmlEncode(sZooName));
                    hashData.Add("zooSelect", GetZooSelect(ixZoo));
                    
                    /* add the hash table as data to the edit template */
                    dlgTemplateEdit.AddTemplateData(row.sRowId, hashData);

                    /* add the data to the delete template as well */
                    dlgTemplateDelete.AddTemplateData(row.sRowId, hashData);
                }
            }
            else
            {
                /* If there are no kiwis, just display a note in a full-width cell */
                CEditableTableRow row = new CEditableTableRow();
                row.sRowId = "none";
                row.AddCellWithColspan("No Kiwis Yet...", nColCount);
                editableTableKiwis.Body.AddRow(row);
            }
            dsKiwis.Dispose();

            /* Add a footer row with icon and text links to the add new kiwi dialog */
            editableTableKiwis.Footer.AddCell(CEditableTable.LinkShowDialogNewIcon(
                                                    sTableId,
                                                    "new",
                                                    "sDataId",
                                                    CommandUrl("new", ixKiwi)));
            editableTableKiwis.Footer.AddCellWithColspan(CEditableTable.LinkShowDialog(
                                                    sTableId,
                                                    "new",
                                                    "sDataId",
                                                    CommandUrl("new", ixKiwi),
                                                    "Add New Kiwi"),
                                                    nColCount - 1);

            /* Associate the dialog templates with the table by name */
            editableTableKiwis.AddDialogTemplate("new", dlgTemplateNew);
            editableTableKiwis.AddDialogTemplate("edit", dlgTemplateEdit);
            editableTableKiwis.AddDialogTemplate("delete", dlgTemplateDelete);

            return editableTableKiwis;
        }

        /* This method builds the template for the add new kiwi dialog */
        protected CDialogTemplate DialogTemplateNew()
        {
            CDialogTemplate dlgTemplateNew = new CDialogTemplate();
            /* There are several dialog formats to choose from */
            dlgTemplateNew.Template = new CDoubleColumnDialog();
            dlgTemplateNew.Template.sTitle = "Add New Kiwi";

            /* FogBugz dialogs post to default.asp via AJAX. To have this form post
             * to the plugin raw page, we need to add the pg and ixPlugin values.
             * Luckily, Forms.UrlAsFormFields will convert a Url into hidden form fields
             * So we can just use api.Url.PluginRawPageUrl */
            CDialogItem itemNewHiddenUrl =
                CDialogItem.HiddenItem(Forms.UrlAsFormFields(api.Url.PluginRawPageUrl()));
            dlgTemplateNew.Template.Items.Add(itemNewHiddenUrl);
            CDialogItem itemNewHiddenAction =
                CDialogItem.HiddenInput(api.AddPluginPrefix("sAction"), "new");
            dlgTemplateNew.Template.Items.Add(itemNewHiddenAction);
            /* include a security action token */
            CDialogItem itemActionToken =
                CDialogItem.HiddenInput(api.AddPluginPrefix("actionToken"), api.Security.GetActionToken());
            dlgTemplateNew.Template.Items.Add(itemActionToken);

            CDialogItem itemNewName =
                new CDialogItem(Forms.TextInput(api.AddPluginPrefix("sKiwiFullName"), ""),
                                "Kiwi Name");
            dlgTemplateNew.Template.Items.Add(itemNewName);

            /* the DateInputOptions class allows you to control the behavior of the FogBugz
             * date picker widget */
            DateInputOptions birthDateOptions = new DateInputOptions();
            birthDateOptions.fAllowFuture = false;
            /* Create a standard Fogbugz date text field and calendar picker pair */
            CDialogItem itemNewBirthDate = new CDialogItem(
                Forms.DateInputCTZ("dtDateOfBirth",
                                   api.AddPluginPrefix("dtDateOfBirth"),
                                   DateTime.Parse("3/25/2009"),
                                   birthDateOptions),
                "Kiwi Birthdate",
                "Enter a date or choose from the calendar (optional)");
            dlgTemplateNew.Template.Items.Add(itemNewBirthDate);

            CDialogItem itemNewZoo = new CDialogItem(
                GetZooSelect(),
                "Zoo Of Residence",
                "Choose the zoo where the kiwi lives from the drop-down");
            dlgTemplateNew.Template.Items.Add(itemNewZoo);

            /* Standard ok and cancel buttons */
            dlgTemplateNew.Template.Items.Add(CEditableTable.DialogItemOkCancel(sTableId));

            return dlgTemplateNew;
        }

        /* This method builds the template for the kiwi edit dialog.
         * FogBugz Dialogs use template variables, enclosed by curly braces.
         * These variables are populated later by the hashtable
         * specified by CDialogTemplate.AddTemplateData */
        protected CDialogTemplate DialogTemplateEdit()
        {
            
            CDialogTemplate dlgTemplateEdit = new CDialogTemplate();
            dlgTemplateEdit.Template = new CDoubleColumnDialog();
            /* names in curly braces are replaced with the otuput of the ToString()
             * method for the corresponding value in the template's data hashtable */
            dlgTemplateEdit.Template.sTitle = "Edit Kiwi id {ixKiwi}: \"{sKiwiFullName}\"";
            CDialogItem itemEditHiddenUrl = 
                CDialogItem.HiddenItem(Forms.UrlAsFormFields(api.Url.PluginRawPageUrl()));
            dlgTemplateEdit.Template.Items.Add(itemEditHiddenUrl);
            CDialogItem itemEditHiddenAction =
                CDialogItem.HiddenInput(api.AddPluginPrefix("sAction"), "edit");
            dlgTemplateEdit.Template.Items.Add(itemEditHiddenAction);
            dlgTemplateEdit.Template.Items.Add(CDialogItem.HiddenInput(
                                                    api.AddPluginPrefix("ixKiwi"),
                                                    "{ixKiwi}"));
            /* include a security action token */
            CDialogItem itemActionToken =
                CDialogItem.HiddenInput(api.AddPluginPrefix("actionToken"), api.Security.GetActionToken());
            dlgTemplateEdit.Template.Items.Add(itemActionToken);

            CDialogItem itemEditName =
                new CDialogItem(Forms.TextInput(api.AddPluginPrefix("sKiwiFullName"),
                                                "{sKiwiFullName}"),
                                "Kiwi Name");
            dlgTemplateEdit.Template.Items.Add(itemEditName);

            DateInputOptions birthDateOptions = new DateInputOptions();
            birthDateOptions.fAllowFuture = false;
            /* Forms.DateInputString takes a string date value, which is neccessary for dialog
             * templates because that value is filled in later by javascript. Forms.DateInput
             * requires a DateTime object which won't work here */
            CDialogItem itemEditBirthDate = new CDialogItem(
                Forms.DateInputString("dtDateOfBirth",
                                      api.AddPluginPrefix("dtDateOfBirth"),
                                      "{dtDateOfBirth}",
                                      birthDateOptions),
                "Kiwi Birthdate",
                "Enter a date or choose from the calendar (optional).");
            dlgTemplateEdit.Template.Items.Add(itemEditBirthDate);

            // we use a placeholder here "zooSelect" which will be replaced by the entire
            // <select><option></option><option></option></select> string because we need to
            // put "selected" as an attribute on the right <option> tag server-side, not in JS
            CDialogItem itemEditZoo = new CDialogItem("{zooSelect}",
                "Zoo Of Residence",
                "Choose the zoo where the kiwi lives from the drop-down.");
            dlgTemplateEdit.Template.Items.Add(itemEditZoo);

            /* Standard ok and cancel buttons */
            dlgTemplateEdit.Template.Items.Add(CEditableTable.DialogItemOkCancel(sTableId));

            return dlgTemplateEdit;
        }

        /* This method builds the template for the delete kiwi dialog */
        protected CDialogTemplate DialogTemplateDelete()
        {
            CDialogTemplate dlgTemplateDelete = new CDialogTemplate();
            dlgTemplateDelete.Template = new CSingleColumnDialog();
            dlgTemplateDelete.Template.sTitle = "Delete Kiwi ID {ixKiwi}: \"{sKiwiFullName}\"";
            dlgTemplateDelete.Template.Items.Add(
                CDialogItem.HiddenInput(api.AddPluginPrefix("ixKiwi"), "{ixKiwi}"));
            CDialogItem itemDeleteHiddenUrl =
                CDialogItem.HiddenItem(Forms.UrlAsFormFields(api.Url.PluginRawPageUrl()));
            dlgTemplateDelete.Template.Items.Add(itemDeleteHiddenUrl);
            CDialogItem itemDeleteHiddenAction =
                CDialogItem.HiddenInput(api.AddPluginPrefix("sAction"), "delete");
            dlgTemplateDelete.Template.Items.Add(itemDeleteHiddenAction);
            /* include a security action token */
            CDialogItem itemActionToken =
                CDialogItem.HiddenInput(api.AddPluginPrefix("actionToken"), api.Security.GetActionToken());
            dlgTemplateDelete.Template.Items.Add(itemActionToken);

            /* DialogItems don't have to be form elements, they can also be simple html */
            dlgTemplateDelete.Template.Items.Add(
                new CDialogItem("Do you want to delete Kiwi {ixKiwi} '{sKiwiFullName}'?")
            );

            /* Standard ok and cancel buttons */
            dlgTemplateDelete.Template.Items.Add(CEditableTable.DialogItemOkCancel(sTableId));

            return dlgTemplateDelete;
        }
        
        /* Render an html drop-down menu from the entries in the Zoo DB table */
        #region Zoo Select

        protected string GetZooSelect()
        {
            return GetZooSelect(-1);
        }

        protected string GetZooSelect(int ixZooSelected)
        {
            /* Fetch all the Zoo names and ids from the database and
             * populate two string arrays */
            string[] zooNames = null;
            string[] zooIxs = null;

            CSelectQuery sq = api.Database.NewSelectQuery(
                api.Database.PluginTableName("Zoo"));
            sq.AddSelect("*");

            DataSet ds = sq.GetDataSet();
            int nZooCount;

            string sSelectInputString = "";

            if (ds.Tables[0] != null && (nZooCount = ds.Tables[0].Rows.Count) > 0)
            {
                zooNames = new string[nZooCount];
                zooIxs = new string[nZooCount];

                for (int i = 0; i < nZooCount; i++)
                {
                    zooNames[i] = HttpUtility.HtmlEncode(ds.Tables[0].Rows[i]["sZooName"].ToString());
                    zooIxs[i] = ds.Tables[0].Rows[i]["ixZoo"].ToString();
                }

                /* like Forms.DateInputString, Forms.SelectInputString is the
                 * version of Forms.SelectInput to be used with dialog templates */
                sSelectInputString = Forms.SelectInputString(api.AddPluginPrefix("ixZoo"),
                                                             Forms.SelectOptions(zooNames,
                                                                                 ixZooSelected.ToString(),
                                                                                 zooIxs));
            }
            else
            {
                api.Notifications.AddAdminNotification("Error in Kiwi Adoption Plugin",
                                                       "There are no zoos defined.");
            }
            ds.Dispose();
            return sSelectInputString;
        }

        #endregion

        /* Generate an informational header, making use of the FogBugz UI library. */
        protected string RenderHeader()
        {
            return String.Format(
                @"{0}
                  <div style=""width: 640px"">
                      <p>
                          This page manages Kiwis in several Zoos. FogBugz UI classes
                          <b>CEditableTable</b> and <b>CDialog</b> are used to easily
                          construct a table and pop-up dialogs which seemlessly match
                          the FogBugz look and feel.
                      </p>
                  </div>",
                FogCreek.FogBugz.UI.PageDisplay.Headline("Adopt a Kiwi"));
        }

        /* these two methods are used to construc the Urls which a user would
         * follow if javascript is disabled (preventing the use of the Dialogs */
        protected string CommandUrl(string sCommand, int ixKiwi)
        {
            return string.Concat(api.Url.PluginPageUrl(),
                                 LinkParameter("sCommand",sCommand),
                                 LinkParameter("ixKiwi",ixKiwi.ToString()));
        }

        protected string LinkParameter(string sName, string sValue)
        {
		    return string.Format("&{0}={1}", api.AddPluginPrefix(sName), sValue);
	    }
    }
}